package javax.swing.plaf.metal;

import java.awt.FontMetrics;
import sun.swing.SwingUtilities2;
import java.awt.Rectangle;
import java.awt.Dimension;
import javax.swing.ButtonModel;
import javax.swing.plaf.UIResource;
import java.awt.Graphics;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicButtonListener;
import javax.swing.AbstractButton;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Color;
import javax.swing.plaf.basic.BasicButtonUI;

public class MetalButtonUI extends BasicButtonUI
{
    protected Color focusColor;
    protected Color selectColor;
    protected Color disabledTextColor;
    private static final Object METAL_BUTTON_UI_KEY;
    
    public MetalButtonUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        MetalButtonUI value = (MetalButtonUI)appContext.get(MetalButtonUI.METAL_BUTTON_UI_KEY);
        if (value == null) {
            value = new MetalButtonUI();
            appContext.put(MetalButtonUI.METAL_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    public void installDefaults(AbstractButton b) {
        super.installDefaults(b);
    }
    
    public void uninstallDefaults(AbstractButton b) {
        super.uninstallDefaults(b);
    }
    
    @Override
    protected BasicButtonListener createButtonListener(AbstractButton b) {
        return super.createButtonListener(b);
    }
    
    protected Color getSelectColor() {
        return this.selectColor = UIManager.getColor(this.getPropertyPrefix() + "select");
    }
    
    protected Color getDisabledTextColor() {
        return this.disabledTextColor = UIManager.getColor(this.getPropertyPrefix() + "disabledText");
    }
    
    protected Color getFocusColor() {
        return this.focusColor = UIManager.getColor(this.getPropertyPrefix() + "focus");
    }
    
    @Override
    public void update(Graphics g, JComponent c) {
        AbstractButton abstractButton = (AbstractButton)c;
        if (c.getBackground() instanceof UIResource && abstractButton.isContentAreaFilled() && c.isEnabled()) {
            ButtonModel model = abstractButton.getModel();
            if (!MetalUtils.isToolBarButton(c)) {
                if (!model.isArmed() && !model.isPressed() && MetalUtils.drawGradient(c, g, "Button.gradient", 0, 0, c.getWidth(), c.getHeight(), true)) {
                    this.paint(g, c);
                    return;
                }
            }
            else if (model.isRollover() && MetalUtils.drawGradient(c, g, "Button.gradient", 0, 0, c.getWidth(), c.getHeight(), true)) {
                this.paint(g, c);
                return;
            }
        }
        super.update(g, c);
    }
    
    @Override
    protected void paintButtonPressed(Graphics graphics, AbstractButton abstractButton) {
        if (abstractButton.isContentAreaFilled()) {
            Dimension size = abstractButton.getSize();
            graphics.setColor(this.getSelectColor());
            graphics.fillRect(0, 0, size.width, size.height);
        }
    }
    
    @Override
    protected void paintFocus(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle, Rectangle rectangle2, Rectangle bounds) {
        Rectangle rectangle3 = new Rectangle();
        String text = abstractButton.getText();
        int n = abstractButton.getIcon() != null;
        if (text != null && !text.equals("")) {
            if (n == 0) {
                rectangle3.setBounds(rectangle2);
            }
            else {
                rectangle3.setBounds(bounds.union(rectangle2));
            }
        }
        else if (!(n == 0)) {
            rectangle3.setBounds(bounds);
        }
        graphics.setColor(this.getFocusColor());
        graphics.drawRect(rectangle3.x - 1, rectangle3.y - 1, rectangle3.width + 1, rectangle3.height + 1);
    }
    
    @Override
    protected void paintText(Graphics graphics, JComponent component, Rectangle rectangle, String text) {
        AbstractButton abstractButton = (AbstractButton)component;
        ButtonModel model = abstractButton.getModel();
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(component, graphics);
        int displayedMnemonicIndex = abstractButton.getDisplayedMnemonicIndex();
        if (model.isEnabled()) {
            graphics.setColor(abstractButton.getForeground());
        }
        else {
            graphics.setColor(this.getDisabledTextColor());
        }
        SwingUtilities2.drawStringUnderlineCharAt(component, graphics, text, displayedMnemonicIndex, rectangle.x, rectangle.y + fontMetrics.getAscent());
    }
    
    static {
        METAL_BUTTON_UI_KEY = new Object();
    }
}
